@@ -6,12 +6,10 @@ import hashlib
 import requests
 import tempfile
 import randomname
-from PIL import Image
 from uuid import uuid4
 
 import logging
 
-import numpy as np
 
 # common/
 nbox_session = requests.Session()
